@@ -29,7 +29,8 @@ pub(crate) fn generate(
         workspace: Some(Workspace {
             members: vec![String::from(".")],
             default_members: vec![String::from(".")],
-            ..Default::default()
+            exclude: Vec::new(),
+            metadata: None,
         }),
         ..empty_manifest()
     }
@@ -64,6 +65,14 @@ fn dependencies(
     });
     deps.insert(String::from("log"), log);
 
+    // we also need lazy_static
+    let lazy_static = Dependency::Detailed(DependencyDetail {
+        version: Some(String::from("1.0")),
+        features: vec![String::from("spin_no_std")],
+        ..empty_dependency_detail()
+    });
+    deps.insert(String::from("lazy_static"), lazy_static);
+
     deps.insert(
         String::from("hotg-rune-core"),
         Dependency::Detailed(rune_project_dependency(
@@ -102,7 +111,7 @@ fn dependencies(
 
 fn proc_block_dependency(
     name: &str,
-    path: &hotg_rune_syntax::ast::Path,
+    path: &hotg_rune_syntax::yaml::Path,
     project: &RuneProject,
     current_dir: &Path,
 ) -> DependencyDetail {
@@ -132,7 +141,7 @@ fn proc_block_dependency(
 }
 
 fn local_proc_block(
-    path: &hotg_rune_syntax::ast::Path,
+    path: &hotg_rune_syntax::yaml::Path,
     current_dir: &Path,
 ) -> DependencyDetail {
     DependencyDetail {
@@ -143,7 +152,7 @@ fn local_proc_block(
 
 fn builtin_proc_block(
     name: &str,
-    path: &hotg_rune_syntax::ast::Path,
+    path: &hotg_rune_syntax::yaml::Path,
     project: &RuneProject,
 ) -> DependencyDetail {
     match project {
@@ -159,7 +168,7 @@ fn builtin_proc_block(
     }
 }
 
-fn is_builtin(path: &hotg_rune_syntax::ast::Path) -> bool {
+fn is_builtin(path: &hotg_rune_syntax::yaml::Path) -> bool {
     path.base == "hotg-ai/rune"
 }
 
@@ -272,8 +281,9 @@ mod tests {
 
         let got = dependencies(&rune, &project, Path::new("."));
 
-        assert_eq!(got.len(), 4);
+        assert_eq!(got.len(), 5);
         assert!(got.contains_key("log"));
+        assert!(got.contains_key("lazy_static"));
         assert!(got.contains_key("hotg-rune-core"));
         assert!(got.contains_key("hotg-rune-proc-blocks"));
         assert!(got.contains_key("hotg-runicos-base-wasm"));
